/* ==================================================================== */
/* Allan CORNET */
/* DIGITEO 2008 - 2009 */
/* ==================================================================== */
#include "../../src/c/dde.h"
#include "stack-c.h"
#include "MALLOC.h"
#include "Scierror.h"
#include "localization.h"
#include "api_scilab.h"
/* ==================================================================== */
int sci_ddeexec(char *fname)
{
  SciErr sciErr;

  int *piAddressVarOne = NULL;
  int iType1 = 0;

  int iParam2Value = -1; /* default value */

  Rhs = Max(0,Rhs);
  CheckRhs(1,2);
  CheckLhs(0,1);

  if (Rhs == 2)
  {
    int *piAddressVarTwo = NULL;
    int iType2 = 0;

    sciErr = getVarAddressFromPosition(pvApiCtx, 2, &piAddressVarTwo);
    if(sciErr.iErr)
    {
      printError(&sciErr, 0);
      return 0;
    }

    sciErr = getVarType(pvApiCtx, piAddressVarTwo, &iType2);
    if(sciErr.iErr)
    {
      printError(&sciErr, 0);
      return 0;
    }

    if (iType2 == sci_matrix)
    {
      double *pdVarTwo = NULL;
      int m2 = 0, n2 = 0;
      sciErr = getMatrixOfDouble(pvApiCtx, piAddressVarTwo,&m2,&n2,&pdVarTwo);
      if(sciErr.iErr)
      {
        printError(&sciErr, 0);
        return 0;
      }

      if ( (m2 == 1) && (n2 == 1) )
      {
        iParam2Value = (int)pdVarTwo[0];
      }
      else
      {
        Scierror(999,_("%s: Wrong size for input argument #%d: A scalar expected.\n"),fname,2);
        return 0;
      }
    }
    else
    {
      Scierror(999,_("%s: Wrong type for input argument #%d: A scalar expected.\n"),fname,2);
      return 0;
    }
  }

  if (Rhs == 1)
  {
    iParam2Value = -1;
  }

  sciErr = getVarAddressFromPosition(pvApiCtx, 1, &piAddressVarOne);
  if(sciErr.iErr)
  {
    printError(&sciErr, 0);
    return 0;
  }

  sciErr = getVarType(pvApiCtx, piAddressVarOne, &iType1);
  if(sciErr.iErr)
  {
    printError(&sciErr, 0);
    return 0;
  }

  if (iType1 == sci_strings)
  {
    BOOL *paramoutBool = NULL;

    int m1 = 0, n1 = 0;
    char *pStVarOne = NULL;
    int lenStVarOne = 0;

    sciErr = getMatrixOfString(pvApiCtx, piAddressVarOne,&m1,&n1,&lenStVarOne,&pStVarOne);
    if(sciErr.iErr)
    {
      printError(&sciErr, 0);
      return 0;
    }

    if ( (m1 != n1) && (n1 != 1) )
    {
      Scierror(999,_("%s: Wrong size for input argument #%d: A string expected.\n"),fname,1);
      return 0;
    }

    pStVarOne = (char*)MALLOC(sizeof(char)*(lenStVarOne + 1));
    if (pStVarOne == NULL)
    {
      Scierror(999,_("%s: No more memory.\n"),fname);
      return 0;
    }

    sciErr = getMatrixOfString(pvApiCtx, piAddressVarOne,&m1,&n1,&lenStVarOne,&pStVarOne);
    if(sciErr.iErr)
    {
      FREE(pStVarOne); pStVarOne = NULL;
      printError(&sciErr, 0);
      return 0;
    }

    paramoutBool = (BOOL*)MALLOC(sizeof(BOOL));
    if (paramoutBool == NULL)
    {
      FREE(pStVarOne); pStVarOne = NULL;
      Scierror(999, _("%s: No more memory.\n"),fname) ;
      return 0;
    }

    if ( DDEExecute(pStVarOne, iParam2Value) )
    {
      paramoutBool[0] = TRUE;
    }
    else
    {
      paramoutBool[0] = FALSE;
    }

    FREE(pStVarOne); pStVarOne = NULL;

    n1 = 1;
    sciErr = createMatrixOfBoolean(pvApiCtx, Rhs + 1, n1, n1, paramoutBool);

    if (paramoutBool)
    {
      FREE(paramoutBool);
      paramoutBool = NULL;
    }

    if(sciErr.iErr)
    {
      printError(&sciErr, 0);
      return 0;
    }

    LhsVar(1) = Rhs+1;

  }
  else
  {
    Scierror(999,_("%s: Wrong type for input argument: String expected.\n"),fname);
  }

  return 0;
}
/* ==================================================================== */

